@@ -1,11 +1,13 @@
 #include "catch.hpp"
 
-#include <functional>
-
 #include <osmium/builder/attr.hpp>
 #include <osmium/memory/buffer.hpp>
 #include <osmium/tags/tags_filter.hpp>
 
+#include <functional>
+#include <iterator>
+#include <string>
+
 TEST_CASE("Tags filter") {
     osmium::memory::Buffer buffer{10240};
 
